@@ -1,17 +1,7 @@
 #! /usr/bin/perl
 
 use strict 'vars';
-use vars qw(
-    $VERSION
-    $NAME
-    $PROMPT
-    $VERBOSE
-    $INTERACTIVE
-    $POLL
-    $ftp
-);
-#use warnings;
-#no warnings 'once';
+use vars qw($VERSION $NAME);
 use Carp 'croak';
 use File::Slurp;
 use File::Temp;
@@ -20,21 +10,29 @@ use Net::FTP;
 use Test::Reporter;
 use Tie::File;
 
-our ($conf, $dist_dir, $file, $files, $got_file, $reporter, $stdout, $TRACK);
+our ($conf, 
+     $dist_dir, 
+     $file, 
+     $files, 
+     $ftp, 
+     $got_file, 
+     $opt, 
+     $reporter, 
+     $stdout, 
+     $track);
 
-$|++;
+$| = 1;
 
 main();
 
 sub main {
-    $VERSION = '0.01_07';
+    $VERSION = '0.01_08';
     $NAME = 'cpantester';
     
-    my $conf = parse_args();
-    
-    weed_out_track( $conf );
-    
-    if (not $POLL) {
+    local ($conf, $opt) = parse_args();
+    usage( $opt );
+
+    if (not $conf->{POLL}) {
         do_verbose( '', 'STDOUT', "--> Mode: non-polling, 1\n" );
         test( $conf, fetch( $conf ), read_track( $conf ) );
     } else {
@@ -46,7 +44,7 @@ sub main {
 	    
 	    test( $conf, fetch( $conf ), read_track( $conf ) ); 
 	    
-	    for (my $sec = $POLL; $sec >= 1; $sec--) {
+	    for (my $sec = $conf->{POLL}; $sec >= 1; $sec--) {
 	        do_verbose( '', 'STDOUT', "$PREFIX $sec " );
 		
 		my $fulllen = (length( $PREFIX ) + 1 + length ( $sec ) + 1 + length( $string ));
@@ -54,7 +52,7 @@ sub main {
 		my $blank = $oldlen - $fulllen;
 		$oldlen = $fulllen;
 		
-		print( $string, ' ' x $blank, "\b" x ($fulllen + $blank) ) if ($VERBOSE && $sec != 1);
+		print( $string, ' ' x $blank, "\b" x ($fulllen + $blank) ) if ($conf->{VERBOSE} && $sec != 1);
 		$blank = 0;
 		
 		sleep 1;
@@ -63,6 +61,8 @@ sub main {
 	}
     }
     
+    weed_out_track( $conf );
+    
     exit 0;
 }
 
@@ -70,15 +70,14 @@ sub test {
     local ($conf, $files, $got_file) = @_;
     local $file;
     
-    local $TRACK = \*TRACK;
-    open ($TRACK, ">>$conf->{track}") or die_mail( "Couldn't open $conf->{track} for writing: $!\n " );
+    open (local $track, ">>$conf->{track}") or die_mail( "Couldn't open $conf->{track} for writing: $!\n " );
 
     for $file (@{$files}) {
         next if ($got_file->{$file} || $file !~ /tar.gz$/);
 	
 	my ($dist) = $file =~ /(.*)\.tar.gz$/;
 	
-	next if do_interactive( "$dist - Skip? [y/N]: ", '^y$', 'print $TRACK "$file\n" unless $got_file->{$file}' );
+	next if do_interactive( "$dist - Skip? [y/N]: ", '^y$', 'print $track "$file\n" unless $got_file->{$file}' );
       
         $ftp->get( $file, "$conf->{dir}/$file" )
           or die_mail( "Couldn't get $file: ", $ftp->message );
@@ -91,74 +90,82 @@ sub test {
 	
     	my @tar = `tar xvvzf $file -C $conf->{dir}`;
 	do_verbose( '', 'STDERR', @tar );
-	die_mail( "$dist: tar xvvzf $file -C $conf->{dir}: $?\n" ) if ($? != 0);
+	die_mail( "$dist: tar xvvzf $file -C $conf->{dir}: $?\n" ) if $?;
 	
 	local $dist_dir = "$conf->{dir}/$dist";
 	
     	unless (chdir ( "$dist_dir" )) {
-	    warn "--> Could not cd to $dist_dir, processing next distribution\n"; 
+	    warn "--> Could not cd to $dist_dir, processing next distribution\n";
+	    print $track "$file\n"; 
 	    $skip = 1; 
 	}
 	
 	if (-e "$dist_dir/Build.PL") {
 	    warn "--> Build.PL exists, processing next distribution\n";
+	    print $track "$file\n";
 	    $skip = 1;
 	}
 
 	next if $skip;
 	
 	next if do_interactive( 'perl Makefile.PL? [Y/n]: ', '^n$' );
+	
+	my $prereqs_notfound;
 	    
 	local $stdout = tmpnam();
 	my $stderr = tmpnam();
 	system( "perl Makefile.PL > $stdout 2>> $stderr" );
 	
-	my $TMPFILE = \*TMPFILE;
-	open ($TMPFILE, $stderr) or die_mail( "Could not open $stderr: $!\n" );
-	while (my $line = <$TMPFILE>) {
+	open (my $tmpfile, $stderr) or die_mail( "Could not open $stderr: $!\n" );
+	while (my $line = <$tmpfile>) {
 	    if (my ($dist, $version) = $line =~ /^Warning: prerequisite (\w+::\w+) (.+) not found\.$/) {
-	        do_verbose( 'warn "--> Prerequisites not found, skipping\n" if $VERBOSE', 'STDOUT', '' );
-	        $skip = 1;
+	        $prereqs_notfound = 1;
             }
         }    
-	close ($TMPFILE) or die_mail( "Could not close $stderr: $!\n" );
-		    
-	die_mail( "$dist: perl Makefile.PL exited on $?\n" ) if ($? != 0);
+	close ($tmpfile) or die_mail( "Could not close $stderr: $!\n" );
 	
-	do_verbose( 'warn "perl Makefile.PL...\n" unless $INTERACTIVE; warn read_file( $stdout );', 'STDOUT', '' );
+	if ($prereqs_notfound) {
+	    do_verbose( 'warn "--> Prerequisites not found, skipping\n" if $conf->{VERBOSE}', 'STDOUT', '' );
+	    print $track "$file\n";
+	    $skip = 1;
+	}
+		    
+	die_mail( "$dist: perl Makefile.PL exited on $?\n" ) if $?;
+	
+	do_verbose( 'warn "perl Makefile.PL...\n" unless $conf->{INTERACTIVE}; warn read_file( $stdout );', 'STDOUT', '' );
 
         next if $skip;
 	
 	next if do_interactive( 'make? [Y/n]: ', '^n$' );
 	
 	my @make = `make`;
-	die_mail( "$dist: make exited on $?\n" ) if ($? != 0);
-	do_verbose( 'warn "make...\n" unless $INTERACTIVE', 'STDOUT', @make );
+	die_mail( "$dist: make exited on $?\n" ) if $?;
+	do_verbose( 'warn "make...\n" unless $conf->{INTERACTIVE}', 'STDOUT', @make );
 	
 	next if do_interactive( 'make test? [Y/n]: ', '^n$' );
 	
 	my @maketest = `make test`;
-	die_mail( "$dist: make test exited on $?\n" ) if $? != 0;
-	do_verbose( 'warn "make test...\n" unless $INTERACTIVE', 'STDOUT', 'STDOUT', @maketest );
+	die_mail( "$dist: make test exited on $?\n" ) if $?;
+	do_verbose( 'warn "make test...\n" unless $conf->{INTERACTIVE}', 'STDOUT', 'STDOUT', @maketest );
 
 	report( $conf, $dist, \@maketest );
 	
 	next if do_interactive( 'make realclean? [Y/n]: ', '^n$' ); 
     	    
         my @makerealclean = `make realclean`;
-        die_mail( "$dist: make realclean exited on $?\n" ) if ($? != 0);
-	do_verbose( 'warn "make realclean...\n" unless $INTERACTIVE', 'STDOUT', @makerealclean );
+        die_mail( "$dist: make realclean exited on $?\n" ) if $?;
+	do_verbose( 'warn "make realclean...\n" unless $conf->{INTERACTIVE}', 'STDOUT', @makerealclean );
 	
 	next if do_interactive( 'rm -rf $dist_dir? [Y/n]: ', '^n$' ); 
     	    
         my @rm = `rm -rf $dist_dir`;
-        die_mail( "$dist: rm -rf exited on $?\n" ) if ($? != 0);
-	do_verbose( 'warn "rm -rf $dist_dir...\n" unless $INTERACTIVE', 'STDOUT', @rm );
+        die_mail( "$dist: rm -rf exited on $?\n" ) if $?;
+	do_verbose( 'warn "rm -rf $dist_dir...\n" unless $conf->{INTERACTIVE}', 'STDOUT', @rm );
 	
-        print $TRACK "$file\n"; # unless $got_file->{$file};
+        print $track "$file\n";
     }
 
-    close ($TRACK) or die_mail( "Couldn't close $conf->{track}: $!\n" );
+    close ($track) or die_mail( "Couldn't close $conf->{track}: $!\n" );
 
     $ftp->quit;
 }
@@ -166,10 +173,10 @@ sub test {
 sub fetch {
     my ($conf) = @_;
 
-    $ftp = Net::FTP->new( $conf->{host}, Debug => $VERBOSE )
+    $ftp = Net::FTP->new( $conf->{host}, Debug => $conf->{VERBOSE} )
       or die_mail( "Couldn't connect to $conf->{host}: ", $ftp->message );
     
-    $ftp->login( 'anonymous','-anonymous@' )
+    $ftp->login( 'anonymous','anonymous@example.com' )
       or die_mail( "Couldn't login: ", $ftp->message ); 
   
     $ftp->binary or die_mail( "Couldn't switch to binary mode: ", $ftp->message );
@@ -206,13 +213,12 @@ sub fetch {
 sub read_track {
     my ($conf) = @_;
     
-    my (%got_file, $TRACK);
-    $TRACK = \*TRACK;
+    my (%got_file, $track);
     
-    open ($TRACK, $conf->{track}) or die_mail( "Couldn't open $conf->{track}: $!\n" );
-    chomp (my @files = <$TRACK>);
+    open (my $track, $conf->{track}) or die_mail( "Couldn't open $conf->{track}: $!\n" );
+    chomp (my @files = <$track>);
     @got_file{ @files } = (1) x @files;
-    close ($TRACK) or die_mail( "Couldn't close $conf->{track}: $!\n" );
+    close ($track) or die_mail( "Couldn't close $conf->{track}: $!\n" );
     
     return \%got_file;
 }
@@ -222,59 +228,44 @@ sub report {
     
     local $reporter = Test::Reporter->new();
 		
-    do_verbose( '$reporter->debug( $VERBOSE )', 'STDERR', '' );
+    do_verbose( '$reporter->debug( $conf->{VERBOSE} )', 'STDERR', '' );
 	    
     $reporter->from( $conf->{mail} );
     $reporter->comments( "Automatically processed by $NAME $VERSION" );
-    $reporter->distribution( $dist );
-		    
-    my $failed = 0;
-	    
-    for my $line (@{$maketest}) {
-        next unless $line =~ /failed/i;
-        $failed = 1 && last;
-    }
-		 
-    if ($failed) { 
-        $reporter->grade('fail');
-    } else { 
-        $reporter->grade('pass');
-    }
-		 
+    $reporter->distribution( $dist );	    
+    $reporter->grade( did_make_fail() );
+	 
     $reporter->send() or die_mail( $reporter->errstr() );
 }
 
 
 sub parse_args {
-    my $conf = usage();
-    
-    my $login   = getlogin;
-    my $homedir = $login =~ /(root)/ ? "/$1" : "/home/$login";
-    
-    my $conf_text = read_file( "$homedir/.cpantesterrc" );
-    %{$conf} = $conf_text =~ /^([^=]+?)\s+=\s+(.+)$/gm;
-    
-    return $conf;
-}
-
-sub usage {
-    my @err = @_;
-    
     $Getopt::Long::autoabbrev = 0;
     $Getopt::Long::ignorecase = 0; 
     
     my (%conf, %opt);
     
-    $PROMPT = "#";
-    
     GetOptions(\%opt, 'h', 'i', 'p=i', 'v', 'V') or $opt{'h'} = 1;
+        
+    my $login   = getlogin;
+    my $homedir = $login =~ /(root)/ ? "/$1" : "/home/$login";
     
-    $INTERACTIVE = $opt{i} ? 1 : 0;
-    $VERBOSE     = $opt{v} ? 1 : 0;
-    $POLL	 = $opt{p} ? $opt{p} : 0;
+    my $conf_text = read_file( "$homedir/.cpantesterrc" );
+    %conf = $conf_text =~ /^([^=]+?)\s+=\s+(.+)$/gm;
+    
+    $conf{PROMPT}      = '#';
+    $conf{INTERACTIVE} = $opt{i} ? 1 : 0;
+    $conf{POLL}        = $opt{p} ? $opt{p} : 0;
+    $conf{VERBOSE}     = $opt{v} ? 1 : 0;
+    
+    return (\%conf, \%opt);
+}
 
-    if ($opt{'h'} || $opt{'V'}) {
-        if ($opt{'h'}) {
+sub usage {
+    my ($opt, @err) = @_;
+    
+    if ($opt->{'h'} || $opt->{'V'}) {
+        if ($opt->{'h'}) {
             print <<"";
 usage: $0 [options]
   -h			this help screen
@@ -292,8 +283,6 @@ usage: $0 [options]
         }
         exit 0;
     }
-    
-    return \%conf;
 }   
 
 sub user_input {
@@ -301,19 +290,29 @@ sub user_input {
     
     my $input;
     do {
-        print "$PROMPT $msg";
+        print "$conf->{PROMPT} $msg";
         chomp ($input = <STDIN>);
     } until ($input =~ /^y$/i || $input =~ /^n$/i || $input eq undef);
     
     return $input;
 }
 
+sub did_make_fail {
+    my (@make_lines) = @_ ;
+ 
+    for my $line (@make_lines) {
+        return 'fail' if $line =~ /failed/;
+    }
+
+    return 'pass';
+}
+
 sub die_mail {
     my @err = @_;
     
     my $login    = getlogin;
     
-    if ($VERBOSE) {
+    if ($conf->{VERBOSE}) {
         warn "--> @err";
         warn "--> Reporting error coincidence via mail to $login", '@localhost', "\n";
     }
@@ -323,10 +322,9 @@ sub die_mail {
     my $to	 = "$login".'@localhost';
     my $subject  = "error: @err";
     
-    my $SENDMAIL = \*SENDMAIL;
-    open ($SENDMAIL, "| $sendmail -t") or die "Could not open | to $sendmail: $!\n";
+    open (my $sendmail, "| $sendmail -t") or die "Could not open | to $sendmail: $!\n";
     
-    my $selold = select ($SENDMAIL);
+    my $selold = select ($sendmail);
     
     print <<"MAIL";
 From: $from
@@ -334,10 +332,8 @@ To: $to
 Subject: $subject
 @err
 MAIL
-    close ($SENDMAIL) or die "Could not close | to sendmail: $!\n";
+    close ($sendmail) or die "Could not close | to sendmail: $!\n";
     select ($selold);
-    
-    #exit ($? != 0) ? $? : -1;
 }
 
 sub weed_out_track {
@@ -352,7 +348,7 @@ sub weed_out_track {
 	    splice (@track, $i, 1);
 	    next;
 	}
-        $file{$track[$i]}++;
+        $file{$track[$i]} = 1;
 	$i++;
     }
     
@@ -362,7 +358,7 @@ sub weed_out_track {
 }
 
 sub do_interactive {
-    if ($INTERACTIVE) {
+    if ($conf->{INTERACTIVE}) {
         my ($prompt, $cond, $eval) = @_;
     
         my $input = user_input( $prompt );
@@ -375,7 +371,7 @@ sub do_interactive {
 }
 
 sub do_verbose {
-    if ($VERBOSE) {
+    if ($conf->{VERBOSE}) {
         my ($eval, $out, @err) = @_;
     
         eval $eval if $eval;
@@ -435,6 +431,10 @@ Upon errors the coincidence will be reported via mail to login@localhost.
 
 Distributions are skipped upon the detection of missing prerequisites.
 
+=head2 System requirements
+
+Tests on Operating systems besides Linux/UNIX aren't supported yet.
+
 =head1 SEE ALSO
 
 L<Test::Reporter>, L<testers.cpan.org>
